@@ -1,11 +1,9 @@
 package js_ast
 
 import (
-	"sort"
 	"strconv"
 
 	"github.com/evanw/esbuild/internal/ast"
-	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/logger"
 )
 
@@ -243,9 +241,10 @@ var OpTable = []OpTableEntry{
 	{"&&=", LAssign, false},
 }
 
-type LocRef struct {
-	Loc logger.Loc
-	Ref Ref
+type Decorator struct {
+	Value            Expr
+	AtLoc            logger.Loc
+	OmitNewlineAfter bool
 }
 
 type PropertyKind uint8
@@ -254,6 +253,7 @@ const (
 	PropertyNormal PropertyKind = iota
 	PropertyGet
 	PropertySet
+	PropertyAutoAccessor
 	PropertySpread
 	PropertyDeclare
 	PropertyClassStaticBlock
@@ -297,7 +297,7 @@ type Property struct {
 	//
 	InitializerOrNil Expr
 
-	Decorators []Expr
+	Decorators []Decorator
 
 	Loc             logger.Loc
 	CloseBracketLoc logger.Loc
@@ -319,17 +319,17 @@ type PropertyBinding struct {
 type Arg struct {
 	Binding      Binding
 	DefaultOrNil Expr
-	Decorators   []Expr
+	Decorators   []Decorator
 
 	// "constructor(public x: boolean) {}"
 	IsTypeScriptCtorField bool
 }
 
 type Fn struct {
-	Name         *LocRef
+	Name         *ast.LocRef
 	Args         []Arg
 	Body         FnBody
-	ArgumentsRef Ref
+	ArgumentsRef ast.Ref
 	OpenParenLoc logger.Loc
 
 	IsAsync     bool
@@ -337,6 +337,9 @@ type Fn struct {
 	HasRestArg  bool
 	HasIfScope  bool
 
+	// See: https://github.com/rollup/rollup/pull/5024
+	HasNoSideEffectsComment bool
+
 	// This is true if the function is a method
 	IsUniqueFormalParameters bool
 }
@@ -347,13 +350,35 @@ type FnBody struct {
 }
 
 type Class struct {
-	Decorators    []Expr
-	Name          *LocRef
+	Decorators    []Decorator
+	Name          *ast.LocRef
 	ExtendsOrNil  Expr
 	Properties    []Property
 	ClassKeyword  logger.Range
 	BodyLoc       logger.Loc
 	CloseBraceLoc logger.Loc
+
+	// If true, property field initializers cannot be assumed to have no side
+	// effects. For example:
+	//
+	//   class Foo {
+	//     static set foo(x) { importantSideEffect(x) }
+	//   }
+	//   class Bar extends Foo {
+	//     foo = 1
+	//   }
+	//
+	// This happens in TypeScript when "useDefineForClassFields" is disabled
+	// because TypeScript (and esbuild) transforms the above class into this:
+	//
+	//   class Foo {
+	//     static set foo(x) { importantSideEffect(x); }
+	//   }
+	//   class Bar extends Foo {
+	//   }
+	//   Bar.foo = 1;
+	//
+	UseDefineForClassFields bool
 }
 
 type ArrayBinding struct {
@@ -378,7 +403,7 @@ func (*BObject) isBinding()     {}
 
 type BMissing struct{}
 
-type BIdentifier struct{ Ref Ref }
+type BIdentifier struct{ Ref ast.Ref }
 
 type BArray struct {
 	Items           []ArrayBinding
@@ -422,7 +447,7 @@ func (*EClass) isExpr()                {}
 func (*EIdentifier) isExpr()           {}
 func (*EImportIdentifier) isExpr()     {}
 func (*EPrivateIdentifier) isExpr()    {}
-func (*EMangledProp) isExpr()          {}
+func (*ENameOfSymbol) isExpr()         {}
 func (*EJSXElement) isExpr()           {}
 func (*EMissing) isExpr()              {}
 func (*ENumber) isExpr()               {}
@@ -433,6 +458,7 @@ func (*EString) isExpr()               {}
 func (*ETemplate) isExpr()             {}
 func (*ERegExp) isExpr()               {}
 func (*EInlinedEnum) isExpr()          {}
+func (*EAnnotation) isExpr()           {}
 func (*EAwait) isExpr()                {}
 func (*EYield) isExpr()                {}
 func (*EIf) isExpr()                   {}
@@ -516,12 +542,14 @@ type EImportMeta struct {
 // These help reduce unnecessary memory allocations
 var BMissingShared = &BMissing{}
 var EMissingShared = &EMissing{}
-var ESuperShared = &ESuper{}
 var ENullShared = &ENull{}
-var EUndefinedShared = &EUndefined{}
+var ESuperShared = &ESuper{}
 var EThisShared = &EThis{}
-var SEmptyShared = &SEmpty{}
+var EUndefinedShared = &EUndefined{}
 var SDebuggerShared = &SDebugger{}
+var SEmptyShared = &SEmpty{}
+var STypeScriptShared = &STypeScript{}
+var STypeScriptSharedWasDeclareClass = &STypeScript{WasDeclareClass: true}
 
 type ENew struct {
 	Target Expr
@@ -541,7 +569,6 @@ const (
 	NormalCall CallKind = iota
 	DirectEval
 	TargetWasOriginallyPropertyAccess
-	InternalPublicFieldCall
 )
 
 type OptionalChain uint8
@@ -634,6 +661,9 @@ type EArrow struct {
 	IsAsync    bool
 	HasRestArg bool
 	PreferExpr bool // Use shorthand if true and "Body" is a single return statement
+
+	// See: https://github.com/rollup/rollup/pull/5024
+	HasNoSideEffectsComment bool
 }
 
 type EFunction struct{ Fn Fn }
@@ -641,7 +671,7 @@ type EFunction struct{ Fn Fn }
 type EClass struct{ Class Class }
 
 type EIdentifier struct {
-	Ref Ref
+	Ref ast.Ref
 
 	// If we're inside a "with" statement, this identifier may be a property
 	// access. In that case it would be incorrect to remove this identifier since
@@ -680,7 +710,7 @@ type EIdentifier struct {
 // "{x: importedNamespace.x}". This separate type forces code to opt-in to
 // doing this instead of opt-out.
 type EImportIdentifier struct {
-	Ref             Ref
+	Ref             ast.Ref
 	PreferQuotedKey bool
 
 	// If true, this was originally an identifier expression such as "foo". If
@@ -693,13 +723,14 @@ type EImportIdentifier struct {
 // methods. It can be used where computed properties can be used, such as
 // EIndex and Property.
 type EPrivateIdentifier struct {
-	Ref Ref
+	Ref ast.Ref
 }
 
 // This represents an internal property name that can be mangled. The symbol
 // referenced by this expression should be a "SymbolMangledProp" symbol.
-type EMangledProp struct {
-	Ref Ref
+type ENameOfSymbol struct {
+	Ref                   ast.Ref
+	HasPropertyKeyComment bool // If true, a preceding comment contains "@__KEY__"
 }
 
 type EJSXElement struct {
@@ -746,9 +777,11 @@ type ESpread struct{ Value Expr }
 // This is used for both strings and no-substitution template literals to reduce
 // the number of cases that need to be checked for string optimization code
 type EString struct {
-	Value          []uint16
-	LegacyOctalLoc logger.Loc
-	PreferTemplate bool
+	Value                 []uint16
+	LegacyOctalLoc        logger.Loc
+	PreferTemplate        bool
+	HasPropertyKeyComment bool // If true, a preceding comment contains "@__KEY__"
+	ContainsUniqueKey     bool // If true, this string must not be wrapped
 }
 
 type TemplatePart struct {
@@ -782,6 +815,24 @@ type EInlinedEnum struct {
 	Comment string
 }
 
+type AnnotationFlags uint8
+
+const (
+	// This is sort of like an IIFE with a "/* @__PURE__ */" comment except it's an
+	// inline annotation on an expression itself without the nested scope. Sometimes
+	// we can't easily introduce a new scope (e.g. if the expression uses "await").
+	CanBeRemovedIfUnusedFlag AnnotationFlags = 1 << iota
+)
+
+func (flags AnnotationFlags) Has(flag AnnotationFlags) bool {
+	return (flags & flag) != 0
+}
+
+type EAnnotation struct {
+	Value Expr
+	Flags AnnotationFlags
+}
+
 type EAwait struct {
 	Value Expr
 }
@@ -869,7 +920,9 @@ type SBlock struct {
 type SEmpty struct{}
 
 // This is a stand-in for a TypeScript type declaration
-type STypeScript struct{}
+type STypeScript struct {
+	WasDeclareClass bool
+}
 
 type SComment struct {
 	Text           string
@@ -890,14 +943,14 @@ type SExportClause struct {
 
 type SExportFrom struct {
 	Items             []ClauseItem
-	NamespaceRef      Ref
+	NamespaceRef      ast.Ref
 	ImportRecordIndex uint32
 	IsSingleLine      bool
 }
 
 type SExportDefault struct {
 	Value       Stmt // May be a SExpr or SFunction or SClass
-	DefaultName LocRef
+	DefaultName ast.LocRef
 }
 
 type ExportStarAlias struct {
@@ -911,7 +964,7 @@ type ExportStarAlias struct {
 
 type SExportStar struct {
 	Alias             *ExportStarAlias
-	NamespaceRef      Ref
+	NamespaceRef      ast.Ref
 	ImportRecordIndex uint32
 }
 
@@ -929,30 +982,33 @@ type SLazyExport struct {
 type SExpr struct {
 	Value Expr
 
-	// This is set to true for automatically-generated expressions that should
-	// not affect tree shaking. For example, calling a function from the runtime
-	// that doesn't have externally-visible side effects.
-	DoesNotAffectTreeShaking bool
+	// This is set to true for automatically-generated expressions that are part
+	// of class syntax lowering. A single class declaration may end up with many
+	// generated expressions after it (e.g. class field initializations, a call
+	// to keep the original value of the "name" property). When this happens we
+	// can't tell that the class is side-effect free anymore because all of these
+	// methods mutate the class. We use this annotation for that instead.
+	IsFromClassOrFnThatCanBeRemovedIfUnused bool
 }
 
 type EnumValue struct {
 	ValueOrNil Expr
 	Name       []uint16
-	Ref        Ref
+	Ref        ast.Ref
 	Loc        logger.Loc
 }
 
 type SEnum struct {
 	Values   []EnumValue
-	Name     LocRef
-	Arg      Ref
+	Name     ast.LocRef
+	Arg      ast.Ref
 	IsExport bool
 }
 
 type SNamespace struct {
 	Stmts    []Stmt
-	Name     LocRef
-	Arg      Ref
+	Name     ast.LocRef
+	Arg      ast.Ref
 	IsExport bool
 }
 
@@ -968,7 +1024,7 @@ type SClass struct {
 
 type SLabel struct {
 	Stmt Stmt
-	Name LocRef
+	Name ast.LocRef
 }
 
 type SIf struct {
@@ -1056,7 +1112,7 @@ type SSwitch struct {
 // Many parts are optional and can be combined in different ways. The only
 // restriction is that you cannot have both a clause and a star namespace.
 type SImport struct {
-	DefaultName *LocRef
+	DefaultName *ast.LocRef
 	Items       *[]ClauseItem
 	StarNameLoc *logger.Loc
 
@@ -1066,7 +1122,7 @@ type SImport struct {
 	// Otherwise: This is an auto-generated Ref for the namespace representing
 	// the imported file. In this case StarLoc is nil. The NamespaceRef is used
 	// when converting this module to a CommonJS module.
-	NamespaceRef Ref
+	NamespaceRef ast.Ref
 
 	ImportRecordIndex uint32
 	IsSingleLine      bool
@@ -1086,8 +1142,14 @@ const (
 	LocalVar LocalKind = iota
 	LocalLet
 	LocalConst
+	LocalUsing
+	LocalAwaitUsing
 )
 
+func (kind LocalKind) IsUsing() bool {
+	return kind >= LocalUsing
+}
+
 type SLocal struct {
 	Decls    []Decl
 	Kind     LocalKind
@@ -1099,11 +1161,11 @@ type SLocal struct {
 }
 
 type SBreak struct {
-	Label *LocRef
+	Label *ast.LocRef
 }
 
 type SContinue struct {
-	Label *LocRef
+	Label *ast.LocRef
 }
 
 type ClauseItem struct {
@@ -1120,7 +1182,7 @@ type ClauseItem struct {
 	OriginalName string
 
 	AliasLoc logger.Loc
-	Name     LocRef
+	Name     ast.LocRef
 }
 
 type Decl struct {
@@ -1128,401 +1190,6 @@ type Decl struct {
 	ValueOrNil Expr
 }
 
-type SymbolKind uint8
-
-const (
-	// An unbound symbol is one that isn't declared in the file it's referenced
-	// in. For example, using "window" without declaring it will be unbound.
-	SymbolUnbound SymbolKind = iota
-
-	// This has special merging behavior. You're allowed to re-declare these
-	// symbols more than once in the same scope. These symbols are also hoisted
-	// out of the scope they are declared in to the closest containing function
-	// or module scope. These are the symbols with this kind:
-	//
-	// - Function arguments
-	// - Function statements
-	// - Variables declared using "var"
-	//
-	SymbolHoisted
-	SymbolHoistedFunction
-
-	// There's a weird special case where catch variables declared using a simple
-	// identifier (i.e. not a binding pattern) block hoisted variables instead of
-	// becoming an error:
-	//
-	//   var e = 0;
-	//   try { throw 1 } catch (e) {
-	//     print(e) // 1
-	//     var e = 2
-	//     print(e) // 2
-	//   }
-	//   print(e) // 0 (since the hoisting stops at the catch block boundary)
-	//
-	// However, other forms are still a syntax error:
-	//
-	//   try {} catch (e) { let e }
-	//   try {} catch ({e}) { var e }
-	//
-	// This symbol is for handling this weird special case.
-	SymbolCatchIdentifier
-
-	// Generator and async functions are not hoisted, but still have special
-	// properties such as being able to overwrite previous functions with the
-	// same name
-	SymbolGeneratorOrAsyncFunction
-
-	// This is the special "arguments" variable inside functions
-	SymbolArguments
-
-	// Classes can merge with TypeScript namespaces.
-	SymbolClass
-
-	// A class-private identifier (i.e. "#foo").
-	SymbolPrivateField
-	SymbolPrivateMethod
-	SymbolPrivateGet
-	SymbolPrivateSet
-	SymbolPrivateGetSetPair
-	SymbolPrivateStaticField
-	SymbolPrivateStaticMethod
-	SymbolPrivateStaticGet
-	SymbolPrivateStaticSet
-	SymbolPrivateStaticGetSetPair
-
-	// Labels are in their own namespace
-	SymbolLabel
-
-	// TypeScript enums can merge with TypeScript namespaces and other TypeScript
-	// enums.
-	SymbolTSEnum
-
-	// TypeScript namespaces can merge with classes, functions, TypeScript enums,
-	// and other TypeScript namespaces.
-	SymbolTSNamespace
-
-	// In TypeScript, imports are allowed to silently collide with symbols within
-	// the module. Presumably this is because the imports may be type-only.
-	SymbolImport
-
-	// Assigning to a "const" symbol will throw a TypeError at runtime
-	SymbolConst
-
-	// Injected symbols can be overridden by provided defines
-	SymbolInjected
-
-	// Properties can optionally be renamed to shorter names
-	SymbolMangledProp
-
-	// This annotates all other symbols that don't have special behavior.
-	SymbolOther
-)
-
-func (kind SymbolKind) IsPrivate() bool {
-	return kind >= SymbolPrivateField && kind <= SymbolPrivateStaticGetSetPair
-}
-
-func (kind SymbolKind) Feature() compat.JSFeature {
-	switch kind {
-	case SymbolPrivateField:
-		return compat.ClassPrivateField
-	case SymbolPrivateMethod:
-		return compat.ClassPrivateMethod
-	case SymbolPrivateGet, SymbolPrivateSet, SymbolPrivateGetSetPair:
-		return compat.ClassPrivateAccessor
-	case SymbolPrivateStaticField:
-		return compat.ClassPrivateStaticField
-	case SymbolPrivateStaticMethod:
-		return compat.ClassPrivateStaticMethod
-	case SymbolPrivateStaticGet, SymbolPrivateStaticSet, SymbolPrivateStaticGetSetPair:
-		return compat.ClassPrivateStaticAccessor
-	default:
-		return 0
-	}
-}
-
-func (kind SymbolKind) IsHoisted() bool {
-	return kind == SymbolHoisted || kind == SymbolHoistedFunction
-}
-
-func (kind SymbolKind) IsHoistedOrFunction() bool {
-	return kind.IsHoisted() || kind == SymbolGeneratorOrAsyncFunction
-}
-
-func (kind SymbolKind) IsFunction() bool {
-	return kind == SymbolHoistedFunction || kind == SymbolGeneratorOrAsyncFunction
-}
-
-func (kind SymbolKind) IsUnboundOrInjected() bool {
-	return kind == SymbolUnbound || kind == SymbolInjected
-}
-
-var InvalidRef Ref = Ref{^uint32(0), ^uint32(0)}
-
-// Files are parsed in parallel for speed. We want to allow each parser to
-// generate symbol IDs that won't conflict with each other. We also want to be
-// able to quickly merge symbol tables from all files into one giant symbol
-// table.
-//
-// We can accomplish both goals by giving each symbol ID two parts: a source
-// index that is unique to the parser goroutine, and an inner index that
-// increments as the parser generates new symbol IDs. Then a symbol map can
-// be an array of arrays indexed first by source index, then by inner index.
-// The maps can be merged quickly by creating a single outer array containing
-// all inner arrays from all parsed files.
-type Ref struct {
-	SourceIndex uint32
-	InnerIndex  uint32
-}
-
-type ImportItemStatus uint8
-
-const (
-	ImportItemNone ImportItemStatus = iota
-
-	// The linker doesn't report import/export mismatch errors
-	ImportItemGenerated
-
-	// The printer will replace this import with "undefined"
-	ImportItemMissing
-)
-
-type SymbolFlags uint16
-
-const (
-	// Certain symbols must not be renamed or minified. For example, the
-	// "arguments" variable is declared by the runtime for every function.
-	// Renaming can also break any identifier used inside a "with" statement.
-	MustNotBeRenamed SymbolFlags = 1 << iota
-
-	// In React's version of JSX, lower-case names are strings while upper-case
-	// names are identifiers. If we are preserving JSX syntax (i.e. not
-	// transforming it), then we need to be careful to name the identifiers
-	// something with a capital letter so further JSX processing doesn't treat
-	// them as strings instead.
-	MustStartWithCapitalLetterForJSX
-
-	// If true, this symbol is the target of a "__name" helper function call.
-	// This call is special because it deliberately doesn't count as a use
-	// of the symbol (otherwise keeping names would disable tree shaking)
-	// so "UseCountEstimate" is not incremented. This flag helps us know to
-	// avoid optimizing this symbol when "UseCountEstimate" is 1 in this case.
-	DidKeepName
-
-	// Sometimes we lower private symbols even if they are supported. For example,
-	// consider the following TypeScript code:
-	//
-	//   class Foo {
-	//     #foo = 123
-	//     bar = this.#foo
-	//   }
-	//
-	// If "useDefineForClassFields: false" is set in "tsconfig.json", then "bar"
-	// must use assignment semantics instead of define semantics. We can compile
-	// that to this code:
-	//
-	//   class Foo {
-	//     constructor() {
-	//       this.#foo = 123;
-	//       this.bar = this.#foo;
-	//     }
-	//     #foo;
-	//   }
-	//
-	// However, we can't do the same for static fields:
-	//
-	//   class Foo {
-	//     static #foo = 123
-	//     static bar = this.#foo
-	//   }
-	//
-	// Compiling these static fields to something like this would be invalid:
-	//
-	//   class Foo {
-	//     static #foo;
-	//   }
-	//   Foo.#foo = 123;
-	//   Foo.bar = Foo.#foo;
-	//
-	// Thus "#foo" must be lowered even though it's supported. Another case is
-	// when we're converting top-level class declarations to class expressions
-	// to avoid the TDZ and the class shadowing symbol is referenced within the
-	// class body:
-	//
-	//   class Foo {
-	//     static #foo = Foo
-	//   }
-	//
-	// This cannot be converted into something like this:
-	//
-	//   var Foo = class {
-	//     static #foo;
-	//   };
-	//   Foo.#foo = Foo;
-	//
-	PrivateSymbolMustBeLowered
-
-	// This is used to remove the all but the last function re-declaration if a
-	// function is re-declared multiple times like this:
-	//
-	//   function foo() { console.log(1) }
-	//   function foo() { console.log(2) }
-	//
-	RemoveOverwrittenFunctionDeclaration
-
-	// This flag is to avoid warning about this symbol more than once. It only
-	// applies to the "module" and "exports" unbound symbols.
-	DidWarnAboutCommonJSInESM
-
-	// If this is present, the symbol could potentially be overwritten. This means
-	// it's not safe to make assumptions about this symbol from the initializer.
-	CouldPotentiallyBeMutated
-
-	// This flags all symbols that were exported from the module using the ES6
-	// "export" keyword, either directly on the declaration or using "export {}".
-	WasExported
-
-	// This means the symbol is a normal function that has no body statements.
-	IsEmptyFunction
-
-	// This means the symbol is a normal function that takes a single argument
-	// and returns that argument.
-	IsIdentityFunction
-)
-
-func (flags SymbolFlags) Has(flag SymbolFlags) bool {
-	return (flags & flag) != 0
-}
-
-// Note: the order of values in this struct matters to reduce struct size.
-type Symbol struct {
-	// This is used for symbols that represent items in the import clause of an
-	// ES6 import statement. These should always be referenced by EImportIdentifier
-	// instead of an EIdentifier. When this is present, the expression should
-	// be printed as a property access off the namespace instead of as a bare
-	// identifier.
-	//
-	// For correctness, this must be stored on the symbol instead of indirectly
-	// associated with the Ref for the symbol somehow. In ES6 "flat bundling"
-	// mode, re-exported symbols are collapsed using MergeSymbols() and renamed
-	// symbols from other files that end up at this symbol must be able to tell
-	// if it has a namespace alias.
-	NamespaceAlias *NamespaceAlias
-
-	// This is the name that came from the parser. Printed names may be renamed
-	// during minification or to avoid name collisions. Do not use the original
-	// name during printing.
-	OriginalName string
-
-	// Used by the parser for single pass parsing. Symbols that have been merged
-	// form a linked-list where the last link is the symbol to use. This link is
-	// an invalid ref if it's the last link. If this isn't invalid, you need to
-	// FollowSymbols to get the real one.
-	Link Ref
-
-	// An estimate of the number of uses of this symbol. This is used to detect
-	// whether a symbol is used or not. For example, TypeScript imports that are
-	// unused must be removed because they are probably type-only imports. This
-	// is an estimate and may not be completely accurate due to oversights in the
-	// code. But it should always be non-zero when the symbol is used.
-	UseCountEstimate uint32
-
-	// This is for generating cross-chunk imports and exports for code splitting.
-	ChunkIndex ast.Index32
-
-	// This is used for minification. Symbols that are declared in sibling scopes
-	// can share a name. A good heuristic (from Google Closure Compiler) is to
-	// assign names to symbols from sibling scopes in declaration order. That way
-	// local variable names are reused in each global function like this, which
-	// improves gzip compression:
-	//
-	//   function x(a, b) { ... }
-	//   function y(a, b, c) { ... }
-	//
-	// The parser fills this in for symbols inside nested scopes. There are three
-	// slot namespaces: regular symbols, label symbols, and private symbols.
-	NestedScopeSlot ast.Index32
-
-	// Boolean values should all be flags instead to save space
-	Flags SymbolFlags
-
-	Kind SymbolKind
-
-	// We automatically generate import items for property accesses off of
-	// namespace imports. This lets us remove the expensive namespace imports
-	// while bundling in many cases, replacing them with a cheap import item
-	// instead:
-	//
-	//   import * as ns from 'path'
-	//   ns.foo()
-	//
-	// That can often be replaced by this, which avoids needing the namespace:
-	//
-	//   import {foo} from 'path'
-	//   foo()
-	//
-	// However, if the import is actually missing then we don't want to report a
-	// compile-time error like we do for real import items. This status lets us
-	// avoid this. We also need to be able to replace such import items with
-	// undefined, which this status is also used for.
-	ImportItemStatus ImportItemStatus
-}
-
-// You should call "MergeSymbols" instead of calling this directly
-func (newSymbol *Symbol) MergeContentsWith(oldSymbol *Symbol) {
-	newSymbol.UseCountEstimate += oldSymbol.UseCountEstimate
-	if oldSymbol.Flags.Has(MustNotBeRenamed) {
-		newSymbol.OriginalName = oldSymbol.OriginalName
-		newSymbol.Flags |= MustNotBeRenamed
-	}
-	if oldSymbol.Flags.Has(MustStartWithCapitalLetterForJSX) {
-		newSymbol.Flags |= MustStartWithCapitalLetterForJSX
-	}
-}
-
-type SlotNamespace uint8
-
-const (
-	SlotDefault SlotNamespace = iota
-	SlotLabel
-	SlotPrivateName
-	SlotMangledProp
-	SlotMustNotBeRenamed
-)
-
-func (s *Symbol) SlotNamespace() SlotNamespace {
-	if s.Kind == SymbolUnbound || s.Flags.Has(MustNotBeRenamed) {
-		return SlotMustNotBeRenamed
-	}
-	if s.Kind.IsPrivate() {
-		return SlotPrivateName
-	}
-	if s.Kind == SymbolLabel {
-		return SlotLabel
-	}
-	if s.Kind == SymbolMangledProp {
-		return SlotMangledProp
-	}
-	return SlotDefault
-}
-
-type SlotCounts [4]uint32
-
-func (a *SlotCounts) UnionMax(b SlotCounts) {
-	for i := range *a {
-		ai := &(*a)[i]
-		bi := b[i]
-		if *ai < bi {
-			*ai = bi
-		}
-	}
-}
-
-type NamespaceAlias struct {
-	Alias        string
-	NamespaceRef Ref
-}
-
 type ScopeKind uint8
 
 const (
@@ -1545,7 +1212,7 @@ func (kind ScopeKind) StopsHoisting() bool {
 }
 
 type ScopeMember struct {
-	Ref Ref
+	Ref ast.Ref
 	Loc logger.Loc
 }
 
@@ -1557,13 +1224,13 @@ type Scope struct {
 	Children  []*Scope
 	Members   map[string]ScopeMember
 	Replaced  []ScopeMember
-	Generated []Ref
+	Generated []ast.Ref
 
 	// The location of the "use strict" directive for ExplicitStrictMode
 	UseStrictLoc logger.Loc
 
 	// This is used to store the ref of the label symbol for ScopeLabel scopes.
-	Label           LocRef
+	Label           ast.LocRef
 	LabelStmtIsLoop bool
 
 	// If a scope contains a direct eval() expression, then none of the symbols
@@ -1662,7 +1329,7 @@ type TSNamespaceScope struct {
 	// generated proxy symbols that represent the property access "x3.y". This
 	// map is unique per namespace block because "x3" is the argument symbol that
 	// is specific to that particular namespace block.
-	LazilyGeneratedProperyAccesses map[string]Ref
+	LazilyGeneratedProperyAccesses map[string]ast.Ref
 
 	// This is specific to this namespace block. It's the argument of the
 	// immediately-invoked function expression that the namespace block is
@@ -1675,7 +1342,7 @@ type TSNamespaceScope struct {
 	//
 	// This variable is "ns2" in the above example. It's the symbol to use when
 	// generating property accesses off of this namespace when it's in scope.
-	ArgRef Ref
+	ArgRef ast.Ref
 
 	// Even though enums are like namespaces and both enums and namespaces allow
 	// implicit references to properties of sibling scopes, they behave like
@@ -1740,25 +1407,6 @@ type TSNamespaceMemberEnumString struct {
 	Value []uint16
 }
 
-type SymbolMap struct {
-	// This could be represented as a "map[Ref]Symbol" but a two-level array was
-	// more efficient in profiles. This appears to be because it doesn't involve
-	// a hash. This representation also makes it trivial to quickly merge symbol
-	// maps from multiple files together. Each file only generates symbols in a
-	// single inner array, so you can join the maps together by just make a
-	// single outer array containing all of the inner arrays. See the comment on
-	// "Ref" for more detail.
-	SymbolsForSource [][]Symbol
-}
-
-func NewSymbolMap(sourceCount int) SymbolMap {
-	return SymbolMap{make([][]Symbol, sourceCount)}
-}
-
-func (sm SymbolMap) Get(ref Ref) *Symbol {
-	return &sm.SymbolsForSource[ref.SourceIndex][ref.InnerIndex]
-}
-
 type ExportsKind uint8
 
 const (
@@ -1833,10 +1481,10 @@ const NSExportPartIndex = uint32(0)
 type AST struct {
 	ModuleTypeData ModuleTypeData
 	Parts          []Part
-	Symbols        []Symbol
+	Symbols        []ast.Symbol
 	ExprComments   map[logger.Loc][]string
 	ModuleScope    *Scope
-	CharFreq       *CharFreq
+	CharFreq       *ast.CharFreq
 
 	// This is internal-only data used for the implementation of Yarn PnP
 	ManifestForYarnPnP Expr
@@ -1850,19 +1498,19 @@ type AST struct {
 	// the linker doesn't mutate this map (cloning a map is slow in Go). Instead the
 	// linker super-imposes relevant information on top in a method call. You should
 	// call "TopLevelSymbolToParts" instead.
-	TopLevelSymbolToPartsFromParser map[Ref][]uint32
+	TopLevelSymbolToPartsFromParser map[ast.Ref][]uint32
 
 	// This contains all top-level exported TypeScript enum constants. It exists
 	// to enable cross-module inlining of constant enums.
-	TSEnums map[Ref]map[string]TSEnumValue
+	TSEnums map[ast.Ref]map[string]TSEnumValue
 
 	// This contains the values of all detected inlinable constants. It exists
 	// to enable cross-module inlining of these constants.
-	ConstValues map[Ref]ConstValue
+	ConstValues map[ast.Ref]ConstValue
 
 	// Properties in here are represented as symbols instead of strings, which
 	// allows them to be renamed to smaller names.
-	MangledProps map[string]Ref
+	MangledProps map[string]ast.Ref
 
 	// Properties in here are existing non-mangled properties in the source code
 	// and must not be used when generating mangled names to avoid a collision.
@@ -1875,7 +1523,7 @@ type AST struct {
 	// These are used when bundling. They are filled in during the parser pass
 	// since we already have to traverse the AST then anyway and the parser pass
 	// is conveniently fully parallelized.
-	NamedImports            map[Ref]NamedImport
+	NamedImports            map[ast.Ref]NamedImport
 	NamedExports            map[string]NamedExport
 	ExportStarImportRecords []uint32
 
@@ -1887,12 +1535,12 @@ type AST struct {
 	TopLevelAwaitKeyword     logger.Range
 	LiveTopLevelAwaitKeyword logger.Range // Excludes top-level await in dead branches
 
-	ExportsRef Ref
-	ModuleRef  Ref
-	WrapperRef Ref
+	ExportsRef ast.Ref
+	ModuleRef  ast.Ref
+	WrapperRef ast.Ref
 
 	ApproximateLineCount  int32
-	NestedScopeSlotCounts SlotCounts
+	NestedScopeSlotCounts ast.SlotCounts
 	HasLazyExport         bool
 
 	// This is a list of CommonJS features. When a file uses CommonJS features,
@@ -1981,104 +1629,6 @@ func ConstValueToExpr(loc logger.Loc, value ConstValue) Expr {
 	panic("Internal error: invalid constant value")
 }
 
-// This is a histogram of character frequencies for minification
-type CharFreq [64]int32
-
-func (freq *CharFreq) Scan(text string, delta int32) {
-	if delta == 0 {
-		return
-	}
-
-	// This matches the order in "DefaultNameMinifier"
-	for i, n := 0, len(text); i < n; i++ {
-		c := text[i]
-		switch {
-		case c >= 'a' && c <= 'z':
-			(*freq)[c-'a'] += delta
-		case c >= 'A' && c <= 'Z':
-			(*freq)[c-('A'-26)] += delta
-		case c >= '0' && c <= '9':
-			(*freq)[c+(52-'0')] += delta
-		case c == '_':
-			(*freq)[62] += delta
-		case c == '$':
-			(*freq)[63] += delta
-		}
-	}
-}
-
-func (freq *CharFreq) Include(other *CharFreq) {
-	for i := 0; i < 64; i++ {
-		(*freq)[i] += (*other)[i]
-	}
-}
-
-type NameMinifier struct {
-	head string
-	tail string
-}
-
-var DefaultNameMinifier = NameMinifier{
-	head: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_$",
-	tail: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_$",
-}
-
-type charAndCount struct {
-	char  string
-	count int32
-	index byte
-}
-
-// This type is just so we can use Go's native sort function
-type charAndCountArray []charAndCount
-
-func (a charAndCountArray) Len() int          { return len(a) }
-func (a charAndCountArray) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
-
-func (a charAndCountArray) Less(i int, j int) bool {
-	ai := a[i]
-	aj := a[j]
-	return ai.count > aj.count || (ai.count == aj.count && ai.index < aj.index)
-}
-
-func (freq *CharFreq) Compile() NameMinifier {
-	// Sort the histogram in descending order by count
-	array := make(charAndCountArray, 64)
-	for i := 0; i < len(DefaultNameMinifier.tail); i++ {
-		array[i] = charAndCount{
-			char:  DefaultNameMinifier.tail[i : i+1],
-			index: byte(i),
-			count: freq[i],
-		}
-	}
-	sort.Sort(array)
-
-	// Compute the identifier start and identifier continue sequences
-	minifier := NameMinifier{}
-	for _, item := range array {
-		if item.char < "0" || item.char > "9" {
-			minifier.head += item.char
-		}
-		minifier.tail += item.char
-	}
-	return minifier
-}
-
-func (minifier *NameMinifier) NumberToMinifiedName(i int) string {
-	j := i % 54
-	name := minifier.head[j : j+1]
-	i = i / 54
-
-	for i > 0 {
-		i--
-		j := i % 64
-		name += minifier.tail[j : j+1]
-		i = i / 64
-	}
-
-	return name
-}
-
 type NamedImport struct {
 	Alias string
 
@@ -2086,7 +1636,7 @@ type NamedImport struct {
 	LocalPartsWithUses []uint32
 
 	AliasLoc          logger.Loc
-	NamespaceRef      Ref
+	NamespaceRef      ast.Ref
 	ImportRecordIndex uint32
 
 	// If true, the alias refers to the entire export namespace object of a
@@ -2101,7 +1651,7 @@ type NamedImport struct {
 }
 
 type NamedExport struct {
-	Ref      Ref
+	Ref      ast.Ref
 	AliasLoc logger.Loc
 }
 
@@ -2124,18 +1674,18 @@ type Part struct {
 	DeclaredSymbols []DeclaredSymbol
 
 	// An estimate of the number of uses of all symbols used within this part.
-	SymbolUses map[Ref]SymbolUse
+	SymbolUses map[ast.Ref]SymbolUse
 
 	// An estimate of the number of uses of all symbols used as the target of
 	// function calls within this part.
-	SymbolCallUses map[Ref]SymbolCallUse
+	SymbolCallUses map[ast.Ref]SymbolCallUse
 
 	// This tracks property accesses off of imported symbols. We don't know
 	// during parsing if an imported symbol is going to be an inlined enum
 	// value or not. This is only known during linking. So we defer adding
 	// a dependency on these imported symbols until we know whether the
 	// property access is an inlined enum value or not.
-	ImportSymbolPropertyUses map[Ref]map[string]SymbolUse
+	ImportSymbolPropertyUses map[ast.Ref]map[string]SymbolUse
 
 	// The indices of the other parts in this file that are needed if this part
 	// is needed.
@@ -2162,7 +1712,7 @@ type Dependency struct {
 }
 
 type DeclaredSymbol struct {
-	Ref        Ref
+	Ref        ast.Ref
 	IsTopLevel bool
 }
 
@@ -2175,62 +1725,6 @@ type SymbolCallUse struct {
 	SingleArgNonSpreadCallCountEstimate uint32
 }
 
-// Returns the canonical ref that represents the ref for the provided symbol.
-// This may not be the provided ref if the symbol has been merged with another
-// symbol.
-func FollowSymbols(symbols SymbolMap, ref Ref) Ref {
-	symbol := symbols.Get(ref)
-	if symbol.Link == InvalidRef {
-		return ref
-	}
-
-	link := FollowSymbols(symbols, symbol.Link)
-
-	// Only write if needed to avoid concurrent map update hazards
-	if symbol.Link != link {
-		symbol.Link = link
-	}
-
-	return link
-}
-
-// Use this before calling "FollowSymbols" from separate threads to avoid
-// concurrent map update hazards. In Go, mutating a map is not threadsafe
-// but reading from a map is. Calling "FollowAllSymbols" first ensures that
-// all mutation is done up front.
-func FollowAllSymbols(symbols SymbolMap) {
-	for sourceIndex, inner := range symbols.SymbolsForSource {
-		for symbolIndex := range inner {
-			FollowSymbols(symbols, Ref{uint32(sourceIndex), uint32(symbolIndex)})
-		}
-	}
-}
-
-// Makes "old" point to "new" by joining the linked lists for the two symbols
-// together. That way "FollowSymbols" on both "old" and "new" will result in
-// the same ref.
-func MergeSymbols(symbols SymbolMap, old Ref, new Ref) Ref {
-	if old == new {
-		return new
-	}
-
-	oldSymbol := symbols.Get(old)
-	if oldSymbol.Link != InvalidRef {
-		oldSymbol.Link = MergeSymbols(symbols, oldSymbol.Link, new)
-		return oldSymbol.Link
-	}
-
-	newSymbol := symbols.Get(new)
-	if newSymbol.Link != InvalidRef {
-		newSymbol.Link = MergeSymbols(symbols, old, newSymbol.Link)
-		return newSymbol.Link
-	}
-
-	oldSymbol.Link = new
-	newSymbol.MergeContentsWith(oldSymbol)
-	return new
-}
-
 // For readability, the names of certain automatically-generated symbols are
 // derived from the file name. For example, instead of the CommonJS wrapper for
 // a file being called something like "require273" it can be called something
